@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -38,36 +38,53 @@
 #define C(x) x, sizeof(x) - 1
 #define S(x) x->str, x->len
 
-#define LUA_IMPORT_INT(x, y) \
-	lua_getfield_literal(L, -1, C(G_STRINGIFY(y))); \
+#define LUA_IMPORT_INT_FROM(_struct, _struct_field, _lua_field) \
+	lua_getfield_literal(L, -1, C(_lua_field)); \
 	if (!lua_isnil(L, -1)) { \
-		x->y = lua_tointeger(L, -1); \
+		_struct->_struct_field = lua_tointeger(L, -1); \
 	} \
 	lua_pop(L, 1);
 
-#define LUA_IMPORT_STR(x, y) \
-	lua_getfield_literal(L, -1, C(G_STRINGIFY(y))); \
+#define LUA_IMPORT_INT(_struct, _struct_field) \
+	LUA_IMPORT_INT_FROM(_struct, _struct_field, G_STRINGIFY(_struct_field))
+
+#define LUA_IMPORT_STR_FROM(_struct, _struct_field, _lua_field) \
+	lua_getfield_literal(L, -1, C(_lua_field)); \
 	if (!lua_isnil(L, -1)) { \
 		size_t s_len; \
 		const char *s = lua_tolstring(L, -1, &s_len); \
-		g_string_assign_len(x->y, s, s_len); \
+		g_string_assign_len(_struct->_struct_field, s, s_len); \
 	} \
 	lua_pop(L, 1);
 
-#define LUA_EXPORT_INT(x, y) \
-	lua_pushinteger(L, x->y); \
-	lua_setfield(L, -2, G_STRINGIFY(y)); 
+#define LUA_IMPORT_STR(_struct, _struct_field) \
+	LUA_IMPORT_STR_FROM(_struct, _struct_field, G_STRINGIFY(_struct_field))
 
-#define LUA_EXPORT_BOOL(x, y) \
-	lua_pushboolean(L, x->y); \
-	lua_setfield(L, -2, G_STRINGIFY(y)); 
+/* and export them */
 
-#define LUA_EXPORT_STR(x, y) \
-	if (x->y->len) { \
-		lua_pushlstring(L, S(x->y)); \
-		lua_setfield(L, -2, G_STRINGIFY(y)); \
+#define LUA_EXPORT_INT_TO(_struct, _struct_field, _lua_field) \
+	lua_pushinteger(L, _struct->_struct_field); \
+	lua_setfield(L, -2, _lua_field); 
+
+#define LUA_EXPORT_INT(_struct, _struct_field) \
+	LUA_EXPORT_INT_TO(_struct, _struct_field, G_STRINGIFY(_struct_field))
+
+#define LUA_EXPORT_BOOL_TO(_struct, _struct_field, _lua_field) \
+	lua_pushboolean(L, _struct->_struct_field); \
+	lua_setfield(L, -2, _lua_field); 
+
+#define LUA_EXPORT_BOOL(_struct, _struct_field) \
+	LUA_EXPORT_BOOL_TO(_struct, _struct_field, G_STRINGIFY(_struct_field))
+
+#define LUA_EXPORT_STR_TO(_struct, _struct_field, _lua_field) \
+	if (_struct->_struct_field->len) { \
+		lua_pushlstring(L, S(_struct->_struct_field)); \
+		lua_setfield(L, -2, _lua_field); \
 	}
 
+#define LUA_EXPORT_STR(_struct, _struct_field) \
+	LUA_EXPORT_STR_TO(_struct, _struct_field, G_STRINGIFY(_struct_field))
+
 static int lua_proto_get_err_packet (lua_State *L) {
 	size_t packet_len;
 	const char *packet_str = luaL_checklstring(L, 1, &packet_len);
@@ -333,6 +350,7 @@ static int lua_proto_append_eof_packet (lua_State *L) {
 static int lua_proto_get_response_packet (lua_State *L) {
 	size_t packet_len;
 	const char *packet_str = luaL_checklstring(L, 1, &packet_len);
+	guint32 server_capabilities = luaL_checkint(L, 2);
 	network_mysqld_auth_response *auth_response;
 	network_packet packet;
 	GString s;
@@ -344,7 +362,7 @@ static int lua_proto_get_response_packet (lua_State *L) {
 	packet.data = &s;
 	packet.offset = 0;
 
-	auth_response = network_mysqld_auth_response_new();
+	auth_response = network_mysqld_auth_response_new(server_capabilities);
 
 	err = err || network_mysqld_proto_get_auth_response(&packet, auth_response);
 	if (err) {
@@ -355,12 +373,14 @@ static int lua_proto_get_response_packet (lua_State *L) {
 	}
 
 	lua_newtable(L);
-	LUA_EXPORT_INT(auth_response, capabilities);
+	LUA_EXPORT_INT_TO(auth_response, client_capabilities, "capabilities");
+	LUA_EXPORT_INT(auth_response, server_capabilities);
 	LUA_EXPORT_INT(auth_response, max_packet_size);
 	LUA_EXPORT_INT(auth_response, charset);
 
 	LUA_EXPORT_STR(auth_response, username);
-	LUA_EXPORT_STR(auth_response, response);
+	LUA_EXPORT_STR_TO(auth_response, auth_plugin_data, "response");
+	LUA_EXPORT_STR(auth_response, auth_plugin_name);
 	LUA_EXPORT_STR(auth_response, database);
 
 	network_mysqld_auth_response_free(auth_response);
@@ -371,18 +391,38 @@ static int lua_proto_get_response_packet (lua_State *L) {
 static int lua_proto_append_response_packet (lua_State *L) {
 	GString *packet;
 	network_mysqld_auth_response *auth_response;
+	gboolean needs_auto_capabilities = TRUE;
+	guint32 server_capabilities;
 
 	luaL_checktype(L, 1, LUA_TTABLE);
 
+	/* extract the server_capabilities first as we need them to create the struct */
+	lua_getfield_literal(L, -1, C("server_capabilities"));
+	if (lua_isnil(L, -1)) {
+		return luaL_error(L, ".server_capabilities has to be set");
+	} else if (lua_isnumber(L, -1)) {
+		server_capabilities = lua_tointeger(L, -1);
+	} else {
+		return luaL_error(L, ".server_capabilities has to be a number");
+	}
+	lua_pop(L, 1);
+
 	packet = g_string_new(NULL);	
-	auth_response = network_mysqld_auth_response_new();
+	auth_response = network_mysqld_auth_response_new(server_capabilities);
+
+	lua_getfield_literal(L, -1, C("capabilities"));
+	if (!lua_isnil(L, -1)) {
+		auth_response->client_capabilities = lua_tointeger(L, -1);
+		needs_auto_capabilities = FALSE;
+	}
+	lua_pop(L, 1);
 
-	LUA_IMPORT_INT(auth_response, capabilities);
 	LUA_IMPORT_INT(auth_response, max_packet_size);
 	LUA_IMPORT_INT(auth_response, charset);
 
 	LUA_IMPORT_STR(auth_response, username);
-	LUA_IMPORT_STR(auth_response, response);
+	LUA_IMPORT_STR_FROM(auth_response, auth_plugin_data, "response");
+	LUA_IMPORT_STR(auth_response, auth_plugin_name);
 	LUA_IMPORT_STR(auth_response, database);
 
 	if (network_mysqld_proto_append_auth_response(packet, auth_response)) {
@@ -390,7 +430,7 @@ static int lua_proto_append_response_packet (lua_State *L) {
 		g_string_free(packet, TRUE);
 
 		luaL_error(L, "to_response_packet() failed");
-        g_string_free(packet, TRUE);
+		g_string_free(packet, TRUE);
 		return 0;
 	}
 	
@@ -435,7 +475,8 @@ static int lua_proto_get_challenge_packet (lua_State *L) {
 	LUA_EXPORT_INT(auth_challenge, charset);
 	LUA_EXPORT_INT(auth_challenge, server_status);
 
-	LUA_EXPORT_STR(auth_challenge, challenge);
+	LUA_EXPORT_STR_TO(auth_challenge, auth_plugin_data, "challenge");
+	LUA_EXPORT_STR(auth_challenge, auth_plugin_name);
 
 	network_mysqld_auth_challenge_free(auth_challenge);
 
@@ -457,7 +498,7 @@ static int lua_proto_append_challenge_packet (lua_State *L) {
 	LUA_IMPORT_INT(auth_challenge, charset);
 	LUA_IMPORT_INT(auth_challenge, server_status);
 
-	LUA_IMPORT_STR(auth_challenge, challenge);
+	LUA_IMPORT_STR_FROM(auth_challenge, auth_plugin_data, "challenge");
 
 	packet = g_string_new(NULL);	
 	network_mysqld_proto_append_auth_challenge(packet, auth_challenge);
