@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -104,6 +104,7 @@
 #include <fcntl.h>
 #include <time.h>
 #include <stdio.h>
+#include <math.h> /* floor() */
 
 #include <errno.h>
 
@@ -152,6 +153,19 @@ typedef int socklen_t;
 #define C(x) x, sizeof(x) - 1
 #define S(x) x->str, x->len
 
+/* backward compat with MySQL pre-5.5.7 */
+#ifndef CLIENT_PLUGIN_AUTH
+#define CLIENT_PLUGIN_AUTH (1 << 19)
+#endif
+
+#ifndef PLUGIN_VERSION
+#ifdef CHASSIS_BUILD_TAG
+#define PLUGIN_VERSION PACKAGE_VERSION "." CHASSIS_BUILD_TAG
+#else
+#define PLUGIN_VERSION PACKAGE_VERSION
+#endif
+#endif
+
 #define HASH_INSERT(hash, key, expr) \
 		do { \
 			GString *hash_value; \
@@ -186,9 +200,61 @@ struct chassis_plugin_config {
 	gint start_proxy;
 
 	network_mysqld_con *listen_con;
+
+	gdouble connect_timeout_dbl; /* exposed in the config as double */
+	gdouble read_timeout_dbl; /* exposed in the config as double */
+	gdouble write_timeout_dbl; /* exposed in the config as double */
 };
 
+/**
+ * handle event-timeouts on the different states
+ *
+ * @note con->state points to the current state
+ *
+ */
+NETWORK_MYSQLD_PLUGIN_PROTO(proxy_timeout) {
+	network_mysqld_con_lua_t *st = con->plugin_con_state;
 
+	if (st == NULL) return NETWORK_SOCKET_ERROR;
+
+	switch (con->state) {
+	case CON_STATE_CONNECT_SERVER:
+		if (con->server) {
+			double timeout = con->connect_timeout.tv_sec +
+				con->connect_timeout.tv_usec / 1000000.0;
+
+			g_debug("%s: connecting to %s timed out after %.2f seconds. Trying another backend.",
+					G_STRLOC,
+					con->server->dst->name->str,
+					timeout);
+
+			st->backend->state = BACKEND_STATE_DOWN;
+			chassis_gtime_testset_now(&st->backend->state_since, NULL);
+			network_socket_free(con->server);
+			con->server = NULL;
+
+			/* stay in this state and let it pick another backend */
+
+			return NETWORK_SOCKET_SUCCESS;
+		}
+		/* fall through */
+	case CON_STATE_SEND_AUTH:
+		if (con->server) {
+			/* we tried to send the auth data to the server, but that timed out.
+			 * send the client and error
+			 */
+			network_mysqld_con_send_error(con->client, C("backend timed out"));
+			con->state = CON_STATE_SEND_AUTH_RESULT;
+			return NETWORK_SOCKET_SUCCESS;
+		}
+		/* fall through */
+	default:
+		/* the client timed out, close the connection */
+		con->state = CON_STATE_ERROR;
+		return NETWORK_SOCKET_SUCCESS;
+	}
+}
+	
 static network_mysqld_lua_stmt_ret proxy_lua_read_query_result(network_mysqld_con *con) {
 	network_socket *send_sock = con->client;
 	network_socket *recv_sock = con->server;
@@ -507,7 +573,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_handshake) {
 	default:
 		g_error("%s.%d: ...", __FILE__, __LINE__);
 		break;
-	} 
+	}
 
 	challenge_packet = g_string_sized_new(packet.data->len); /* the packet we generate will be likely as large as the old one. should save some reallocs */
 	network_mysqld_proto_append_auth_challenge(challenge_packet, challenge);
@@ -519,6 +585,9 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_handshake) {
 	g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
 
 	/* copy the pack to the client */
+	g_assert(con->client->challenge == NULL);
+	con->client->challenge = network_mysqld_auth_challenge_copy(challenge);
+	
 	con->state = CON_STATE_SEND_HANDSHAKE;
 
 	return NETWORK_SOCKET_SUCCESS;
@@ -622,6 +691,7 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 	int err = 0;
 	gboolean free_client_packet = TRUE;
 	network_mysqld_con_lua_t *st = con->plugin_con_state;
+	gboolean got_all_data = TRUE;
 
 	recv_sock = con->client;
 	send_sock = con->server;
@@ -632,148 +702,223 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth) {
 	err = err || network_mysqld_proto_skip_network_header(&packet);
 	if (err) return NETWORK_SOCKET_ERROR;
 
-	auth = network_mysqld_auth_response_new();
-
-	err = err || network_mysqld_proto_get_auth_response(&packet, auth);
-
-	if (err) {
-		network_mysqld_auth_response_free(auth);
-		return NETWORK_SOCKET_ERROR;
-	}
-	if (!(auth->capabilities & CLIENT_PROTOCOL_41)) {
-		/* should use packet-id 0 */
-		network_mysqld_queue_append(con->client, con->client->send_queue, C("\xff\xd7\x07" "4.0 protocol is not supported"));
-		network_mysqld_auth_response_free(auth);
-		return NETWORK_SOCKET_ERROR;
-	}
-
- 	con->client->response = auth;
-
-	g_string_assign_len(con->client->default_db, S(auth->database));
-
-	/**
-	 * looks like we finished parsing, call the lua function
+	/* assume that we may get called twice:
+	 *
+	 * 1. for the initial packet
+	 * 2. for the win-auth extra data
+	 *
+	 * this is detected by con->client->response being NULL
 	 */
-	switch (proxy_lua_read_auth(con)) {
-	case PROXY_SEND_RESULT:
-		con->state = CON_STATE_SEND_AUTH_RESULT;
 
-		break;
-	case PROXY_SEND_INJECTION: {
-		injection *inj;
+	if (con->client->response == NULL) {
+		auth = network_mysqld_auth_response_new(con->client->challenge->capabilities);
 
-		/* replace the client challenge that is sent to the server */
-		inj = g_queue_pop_head(st->injected.queries);
+		err = err || network_mysqld_proto_get_auth_response(&packet, auth);
 
-		network_mysqld_queue_append(send_sock, send_sock->send_queue, S(inj->query));
+		if (err) {
+			network_mysqld_auth_response_free(auth);
+			return NETWORK_SOCKET_ERROR;
+		}
+		if (!(auth->client_capabilities & CLIENT_PROTOCOL_41)) {
+			/* should use packet-id 0 */
+			network_mysqld_queue_append(con->client, con->client->send_queue, C("\xff\xd7\x07" "4.0 protocol is not supported"));
+			network_mysqld_auth_response_free(auth);
+			return NETWORK_SOCKET_ERROR;
+		}
 
-		injection_free(inj);
+ 		con->client->response = auth;
 
-		con->state = CON_STATE_SEND_AUTH;
+		g_string_assign_len(con->client->default_db, S(auth->database));
 
-		break; }
-	case PROXY_NO_DECISION:
-		/* if we don't have a backend (con->server), we just ack the client auth
+		/* client and server support auth-plugins and the client uses
+		 * win-auth, we may have more data to read from the client
 		 */
-		if (!con->server) {
+		if ((auth->server_capabilities & CLIENT_PLUGIN_AUTH) &&
+		    (auth->client_capabilities & CLIENT_PLUGIN_AUTH) &&
+		    (strleq(S(auth->auth_plugin_name), C("authentication_windows_client"))) &&
+		    (auth->auth_plugin_data->len == 255)) {
+#if 1
+			/**
+			 * FIXME: the 2-packet win-auth protocol enhancements aren't properly tested yet.
+			 * therefore they are disabled for now.
+			 */
+			g_string_free(g_queue_pop_head(con->client->recv_queue->chunks), TRUE);
+
+			network_mysqld_con_send_error(con->client, C("long packets for windows-authentication aren't completely handled yet. Please use another auth-method for now."));
+
+			return NETWORK_SOCKET_ERROR;
+#else
+
+			got_all_data = FALSE; /* strip the last byte as it is used for extra signaling that we should ignore */
+			g_string_truncate(auth->auth_plugin_data, auth->auth_plugin_data->len - 1);
+#endif
+		} else {
+			got_all_data = TRUE;
+		}
+	} else {
+		GString *auth_data;
+		gsize auth_data_len;
+
+		/* this is the 2nd round. We don't expect more data */
+		got_all_data = TRUE;
+
+		/* get all the data from the packet and append it to the auth_plugin_data */
+		auth_data_len = packet.data->len - 4;
+		auth_data = g_string_sized_new(auth_data_len);
+		network_mysqld_proto_get_gstring_len(&packet, auth_data_len, auth_data);
+
+		g_string_append_len(con->client->response->auth_plugin_data, S(auth_data));
+
+		g_string_free(auth_data, TRUE);
+	}
+
+	if (got_all_data) {
+		/**
+		 * looks like we finished parsing, call the lua function
+		 */
+		switch (proxy_lua_read_auth(con)) {
+		case PROXY_SEND_RESULT:
 			con->state = CON_STATE_SEND_AUTH_RESULT;
 
-			network_mysqld_con_send_ok(recv_sock);
-
 			break;
-		}
-		/* if the server-side of the connection is already up and authed
-		 * we send a COM_CHANGE_USER to reauth the connection and remove
-		 * all temp-tables and session-variables
-		 *
-		 * for performance reasons this extra reauth can be disabled. But
-		 * that leaves temp-tables on the connection.
-		 */
-		if (con->server->is_authed) {
-			if (config->pool_change_user) {
-				GString *com_change_user = g_string_new(NULL);
+		case PROXY_SEND_INJECTION: {
+			injection *inj;
 
-				/* copy incl. the nul */
-				g_string_append_c(com_change_user, COM_CHANGE_USER);
-				g_string_append_len(com_change_user, con->client->response->username->str, con->client->response->username->len + 1); /* nul-term */
+			/* replace the client challenge that is sent to the server */
+			inj = g_queue_pop_head(st->injected.queries);
 
-				g_assert_cmpint(con->client->response->response->len, <, 250);
+			network_mysqld_queue_append(send_sock, send_sock->send_queue, S(inj->query));
 
-				g_string_append_c(com_change_user, (con->client->response->response->len & 0xff));
-				g_string_append_len(com_change_user, S(con->client->response->response));
+			injection_free(inj);
 
-				g_string_append_len(com_change_user, con->client->default_db->str, con->client->default_db->len + 1);
-				
-				network_mysqld_queue_append(
-						send_sock,
-						send_sock->send_queue, 
-						S(com_change_user));
-
-				/**
-				 * the server is already authenticated, the client isn't
-				 *
-				 * transform the auth-packet into a COM_CHANGE_USER
-				 */
-
-				g_string_free(com_change_user, TRUE);
-			
-				con->state = CON_STATE_SEND_AUTH;
-			} else {
-				GString *auth_resp;
-
-				/* check if the username and client-scramble are the same as in the previous authed
-				 * connection */
-
-				auth_resp = g_string_new(NULL);
-
-				con->state = CON_STATE_SEND_AUTH_RESULT;
-
-				if (!g_string_equal(con->client->response->username, con->server->response->username) ||
-				    !g_string_equal(con->client->response->response, con->server->response->response)) {
-					network_mysqld_err_packet_t *err_packet;
-
-					err_packet = network_mysqld_err_packet_new();
-					g_string_assign_len(err_packet->errmsg, C("(proxy-pool) login failed"));
-					g_string_assign_len(err_packet->sqlstate, C("28000"));
-					err_packet->errcode = ER_ACCESS_DENIED_ERROR;
-
-					network_mysqld_proto_append_err_packet(auth_resp, err_packet);
-
-					network_mysqld_err_packet_free(err_packet);
-				} else {
-					network_mysqld_ok_packet_t *ok_packet;
-
-					ok_packet = network_mysqld_ok_packet_new();
-					ok_packet->server_status = SERVER_STATUS_AUTOCOMMIT;
-
-					network_mysqld_proto_append_ok_packet(auth_resp, ok_packet);
-					
-					network_mysqld_ok_packet_free(ok_packet);
-				}
-
-				network_mysqld_queue_append(recv_sock, recv_sock->send_queue, 
-						S(auth_resp));
-
-				g_string_free(auth_resp, TRUE);
-			}
-		} else {
-			network_mysqld_queue_append_raw(send_sock, send_sock->send_queue, packet.data);
 			con->state = CON_STATE_SEND_AUTH;
 
-			free_client_packet = FALSE; /* the packet.data is now part of the send-queue, don't free it further down */
+			break; }
+		case PROXY_NO_DECISION:
+			/* if we don't have a backend (con->server), we just ack the client auth
+			 */
+			if (!con->server) {
+				con->state = CON_STATE_SEND_AUTH_RESULT;
+
+				network_mysqld_con_send_ok(recv_sock);
+
+				break;
+			}
+			/* if the server-side of the connection is already up and authed
+			 * we send a COM_CHANGE_USER to reauth the connection and remove
+			 * all temp-tables and session-variables
+			 *
+			 * for performance reasons this extra reauth can be disabled. But
+			 * that leaves temp-tables on the connection.
+			 */
+			if (con->server->is_authed) {
+				if (config->pool_change_user) {
+					GString *com_change_user = g_string_new(NULL);
+
+					/* copy incl. the nul */
+					g_string_append_c(com_change_user, COM_CHANGE_USER);
+					g_string_append_len(com_change_user, con->client->response->username->str, con->client->response->username->len + 1); /* nul-term */
+
+					g_assert_cmpint(con->client->response->auth_plugin_data->len, <, 250);
+
+					g_string_append_c(com_change_user, (con->client->response->auth_plugin_data->len & 0xff));
+					g_string_append_len(com_change_user, S(con->client->response->auth_plugin_data));
+
+					g_string_append_len(com_change_user, con->client->default_db->str, con->client->default_db->len + 1);
+
+					network_mysqld_proto_append_int16(com_change_user, con->client->response->charset);
+
+					if (con->client->challenge->capabilities & CLIENT_PLUGIN_AUTH) {
+						g_string_append_len(com_change_user, con->client->response->auth_plugin_name->str, con->client->response->auth_plugin_name->len + 1);
+					}
+
+					network_mysqld_queue_append(
+							send_sock,
+							send_sock->send_queue, 
+							S(com_change_user));
+
+					/* we just injected a com_change_user packet so let's set the flag to track it on the connection */
+					st->is_in_com_change_user = TRUE;
+
+					/**
+					 * the server is already authenticated, the client isn't
+					 *
+					 * transform the auth-packet into a COM_CHANGE_USER
+					 */
+
+					g_string_free(com_change_user, TRUE);
+				
+					con->state = CON_STATE_SEND_AUTH;
+				} else {
+					GString *auth_resp;
+
+					/* check if the username and client-scramble are the same as in the previous authed
+					 * connection */
+
+					auth_resp = g_string_new(NULL);
+
+					con->state = CON_STATE_SEND_AUTH_RESULT;
+
+					if (!g_string_equal(con->client->response->username, con->server->response->username) ||
+					    !g_string_equal(con->client->response->auth_plugin_data, con->server->response->auth_plugin_data)) {
+						network_mysqld_err_packet_t *err_packet;
+
+						err_packet = network_mysqld_err_packet_new();
+						g_string_assign_len(err_packet->errmsg, C("(proxy-pool) login failed"));
+						g_string_assign_len(err_packet->sqlstate, C("28000"));
+						err_packet->errcode = ER_ACCESS_DENIED_ERROR;
+
+						network_mysqld_proto_append_err_packet(auth_resp, err_packet);
+
+						network_mysqld_err_packet_free(err_packet);
+					} else {
+						network_mysqld_ok_packet_t *ok_packet;
+
+						ok_packet = network_mysqld_ok_packet_new();
+						ok_packet->server_status = SERVER_STATUS_AUTOCOMMIT;
+
+						network_mysqld_proto_append_ok_packet(auth_resp, ok_packet);
+						
+						network_mysqld_ok_packet_free(ok_packet);
+					}
+
+					network_mysqld_queue_append(recv_sock, recv_sock->send_queue, 
+							S(auth_resp));
+					
+					/* the server side of connection is already up and authed and we have checked that
+					 * the username and client-scramble are the same as in the previous authed connection.
+					 * the auth phase is over so we need to reset the packet-id sequence
+					 */
+					network_mysqld_queue_reset(send_sock);
+					network_mysqld_queue_reset(recv_sock);
+
+					g_string_free(auth_resp, TRUE);
+				}
+			} else {
+				network_mysqld_queue_append_raw(send_sock, send_sock->send_queue, packet.data);
+				con->state = CON_STATE_SEND_AUTH;
+
+				free_client_packet = FALSE; /* the packet.data is now part of the send-queue, don't free it further down */
+			}
+
+			break;
+		default:
+			g_assert_not_reached();
+			break;
 		}
 
-		break;
-	default:
-		g_assert_not_reached();
-		break;
-	}
-
-	if (free_client_packet) {
-		g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
+		if (free_client_packet) {
+			g_string_free(g_queue_pop_tail(recv_sock->recv_queue->chunks), TRUE);
+		} else {
+			/* just remove the link to the packet, the packet itself is part of the next queue already */
+			g_queue_pop_tail(recv_sock->recv_queue->chunks);
+		}
 	} else {
-		/* just remove the link to the packet, the packet itself is part of the next queue already */
-		g_queue_pop_tail(recv_sock->recv_queue->chunks);
+		/* move the packet from the recv-queue to the send-queue AS IS */
+
+		network_mysqld_queue_append_raw(send_sock, send_sock->send_queue,
+				g_queue_pop_tail(recv_sock->recv_queue->chunks));
+		/* stay in this state and read the next packet */
 	}
 
 	return NETWORK_SOCKET_SUCCESS;
@@ -951,6 +1096,74 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth_result) {
 	return NETWORK_SOCKET_SUCCESS;
 }
 
+NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_auth_old_password) {
+	network_socket *recv_sock, *send_sock;
+	network_packet packet;
+	guint32 packet_len;
+	network_mysqld_con_lua_t *st = con->plugin_con_state;
+
+	/* move the packet to the send queue */
+
+	recv_sock = con->client;
+	send_sock = con->server;
+
+	if (NULL == con->server) {
+		/**
+		 * we have to auth against same backend as we did before
+		 * but the user changed it
+		 */
+
+		network_mysqld_con_send_error(con->client, C("(lua) read-auth-old-password failed as backend_ndx got reset."));
+		con->state = CON_STATE_SEND_ERROR;
+		return NETWORK_SOCKET_SUCCESS;
+	}
+
+	packet.data = g_queue_peek_head(recv_sock->recv_queue->chunks);
+	packet.offset = 0;
+
+	packet_len = network_mysqld_proto_get_packet_len(packet.data);
+
+	if ((strleq(S(con->auth_switch_to_method), C("authentication_windows_client"))) &&
+	    (con->auth_switch_to_round == 0) &&
+	    (packet_len == 255)) {
+#if 1
+		/**
+		 * FIXME: the 2-packet win-auth protocol enhancements aren't properly tested yet.
+		 * therefore they are disabled for now.
+		 */
+		g_string_free(g_queue_pop_head(recv_sock->recv_queue->chunks), TRUE);
+
+		network_mysqld_con_send_error(recv_sock, C("long packets for windows-authentication aren't completely handled yet. Please use another auth-method for now."));
+
+		con->state = CON_STATE_SEND_ERROR;
+#else
+		con->auth_switch_to_round++;
+		/* move the packet to the send-queue
+		 */
+		network_mysqld_queue_append_raw(send_sock, send_sock->send_queue,
+				g_queue_pop_head(recv_sock->recv_queue->chunks));
+
+		/* stay in this state and read the next packet too */
+#endif
+	} else {
+		/* let's check if the proxy plugin injected a com_change_user packet so we
+		 * need to fix the packet-id
+		 */
+		if (st->is_in_com_change_user) {
+			network_mysqld_proto_set_packet_id(packet.data, send_sock->last_packet_id + 1);
+		}
+
+		/* move the packet to the send-queue
+		 */
+		network_mysqld_queue_append_raw(send_sock, send_sock->send_queue,
+				g_queue_pop_head(recv_sock->recv_queue->chunks));
+
+		con->state = CON_STATE_SEND_AUTH_OLD_PASSWORD;
+	}
+
+	return NETWORK_SOCKET_SUCCESS;
+}
+
 static network_mysqld_lua_stmt_ret proxy_lua_read_query(network_mysqld_con *con) {
 	network_mysqld_con_lua_t *st = con->plugin_con_state;
 	network_socket *recv_sock = con->client;
@@ -1112,6 +1325,11 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_query) {
 	recv_sock = con->client;
 	st->injected.sent_resultset = 0;
 
+	/* we already passed the CON_STATE_READ_AUTH_OLD_PASSWORD phase and sent all packets
+	 * to the client so we need to set the COM_CHANGE_USER flag back to FALSE
+	 */
+	st->is_in_com_change_user = FALSE;
+
 	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query::enter_lua");
 	ret = proxy_lua_read_query(con);
 	NETWORK_MYSQLD_CON_TRACK_TIME(con, "proxy::ready_query::leave_lua");
@@ -1456,6 +1674,34 @@ static network_mysqld_lua_stmt_ret proxy_lua_connect_server(network_mysqld_con *
 		
 				/* send packet-id 0 */
 				network_mysqld_con_send_error(con->client, C("(lua) handling proxy.response failed, check error-log"));
+			} else {
+				network_queue *q;
+				network_packet packet;
+				int err = 0;
+				guint8 packet_type;
+
+				/* we should have a auth-packet or a err-packet in the queue */
+				q = con->client->send_queue;
+
+				packet.data = g_queue_peek_head(q->chunks);
+				packet.offset = 0;
+
+				err = err || network_mysqld_proto_skip_network_header(&packet);
+				err = err || network_mysqld_proto_peek_int8(&packet, &packet_type);
+				if (!err && packet_type == 0x0a) {
+					network_mysqld_auth_challenge *challenge;
+
+					challenge = network_mysqld_auth_challenge_new();
+
+					err = err || network_mysqld_proto_get_auth_challenge(&packet, challenge);
+
+					if (!err) {
+						g_assert(con->client->challenge == NULL); /* make sure we don't leak memory */
+						con->client->challenge = challenge;
+					} else {
+						network_mysqld_auth_challenge_free(challenge);
+					}
+				}
 			}
 
 			break;
@@ -1500,6 +1746,8 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 	if (con->server) {
 		switch (network_socket_connect_finish(con->server)) {
 		case NETWORK_SOCKET_SUCCESS:
+			/* increment the connected clients value only if we connected successfully */
+			st->backend->connected_clients++;
 			break;
 		case NETWORK_SOCKET_ERROR:
 		case NETWORK_SOCKET_ERROR_RETRY:
@@ -1624,8 +1872,6 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 	if (NULL == con->server) {
 		con->server = network_socket_new();
 		network_address_copy(con->server->dst, st->backend->addr);
-	
-		st->backend->connected_clients++;
 
 		switch(network_socket_connect(con->server)) {
 		case NETWORK_SOCKET_ERROR_RETRY:
@@ -1633,6 +1879,8 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 			 * call getsockopt() to see if we are done */
 			return NETWORK_SOCKET_ERROR_RETRY;
 		case NETWORK_SOCKET_SUCCESS:
+			/* increment the connected clients value only if we connected successfully */
+			st->backend->connected_clients++;
 			break;
 		default:
 			g_message("%s.%d: connecting to backend (%s) failed, marking it as down for ...", 
@@ -1669,7 +1917,10 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 				S(auth_packet));
 
 		g_string_free(auth_packet, TRUE);
-		
+
+		g_assert(con->client->challenge == NULL);
+		con->client->challenge = network_mysqld_auth_challenge_copy(con->server->challenge);
+
 		con->state = CON_STATE_SEND_HANDSHAKE;
 
 		/**
@@ -1680,8 +1931,23 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_connect_server) {
 	return NETWORK_SOCKET_SUCCESS;
 }
 
+/**
+ * convert a double into a timeval
+ */
+static gboolean
+timeval_from_double(struct timeval *dst, double t) {
+	g_return_val_if_fail(dst != NULL, FALSE);
+	g_return_val_if_fail(t >= 0, FALSE);
+
+	dst->tv_sec = floor(t);
+	dst->tv_usec = floor((t - dst->tv_sec) * 1000000);
+
+	return TRUE;
+}
+
 NETWORK_MYSQLD_PLUGIN_PROTO(proxy_init) {
 	network_mysqld_con_lua_t *st = con->plugin_con_state;
+	chassis_plugin_config *config = con->config;
 
 	g_assert(con->plugin_con_state == NULL);
 
@@ -1691,6 +1957,22 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_init) {
 	
 	con->state = CON_STATE_CONNECT_SERVER;
 
+	/* set the connection specific timeouts
+	 *
+	 * TODO: expose these settings at runtime
+	 */
+	if (config->connect_timeout_dbl >= 0) {
+		timeval_from_double(&con->connect_timeout, config->connect_timeout_dbl);
+	}
+	if (config->read_timeout_dbl >= 0) {
+		timeval_from_double(&con->read_timeout, config->read_timeout_dbl);
+	}
+	if (config->write_timeout_dbl >= 0) {
+		timeval_from_double(&con->write_timeout, config->write_timeout_dbl);
+	}
+
+
+
 	return NETWORK_SOCKET_SUCCESS;
 }
 
@@ -1857,9 +2139,14 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_local_infile_data) {
 	g_assert_cmpint(com_query->state, ==, PARSE_COM_QUERY_LOCAL_INFILE_DATA);
 
 	query_result = network_mysqld_proto_get_query_result(&packet, con);
+
+	/* set the testing flag for all data received or not */ 
+	con->local_file_data_is_finished = (query_result == 1);
+
 	if (query_result == -1) return NETWORK_SOCKET_ERROR; /* something happend, let's get out of here */
 
 	if (con->server) {
+		/* we haven't received all data from load data infile, so let's continue reading and writing to the backend */
 		network_mysqld_queue_append_raw(send_sock, send_sock->send_queue,
 				g_queue_pop_tail(recv_sock->recv_queue->chunks));
 	} else {
@@ -1872,8 +2159,8 @@ NETWORK_MYSQLD_PLUGIN_PROTO(proxy_read_local_infile_data) {
 		while ((s = g_queue_pop_head(recv_sock->recv_queue->chunks))) g_string_free(s, TRUE);
 	}
 
-	if (query_result == 1) { /* we have everything, send it to the backend */
-		if (con->server) {
+	if (query_result == 1) {
+		if (con->server) { /* we have received all data, lets move forward reading the result from the server */
 			con->state = CON_STATE_SEND_LOCAL_INFILE_DATA;
 		} else {
 			network_mysqld_con_send_ok(con->client);
@@ -1944,6 +2231,7 @@ int network_mysqld_proxy_connection_init(network_mysqld_con *con) {
 	con->plugins.con_read_handshake            = proxy_read_handshake;
 	con->plugins.con_read_auth                 = proxy_read_auth;
 	con->plugins.con_read_auth_result          = proxy_read_auth_result;
+	con->plugins.con_read_auth_old_password	   = proxy_read_auth_old_password;
 	con->plugins.con_read_query                = proxy_read_query;
 	con->plugins.con_read_query_result         = proxy_read_query_result;
 	con->plugins.con_send_query_result         = proxy_send_query_result;
@@ -1951,6 +2239,7 @@ int network_mysqld_proxy_connection_init(network_mysqld_con *con) {
 	con->plugins.con_read_local_infile_result = proxy_read_local_infile_result;
 	con->plugins.con_send_local_infile_result = proxy_send_local_infile_result;
 	con->plugins.con_cleanup                   = proxy_disconnect_client;
+	con->plugins.con_timeout                   = proxy_timeout;
 
 	return 0;
 }
@@ -1973,6 +2262,11 @@ chassis_plugin_config * network_mysqld_proxy_plugin_new(void) {
 	config->pool_change_user = 1; /* issue a COM_CHANGE_USER to cleanup the connection 
 					 when we get back the connection from the pool */
 
+	/* use negative values as defaults to make them ignored */
+	config->connect_timeout_dbl = -1.0;
+	config->read_timeout_dbl = -1.0;
+	config->write_timeout_dbl = -1.0;
+
 	return config;
 }
 
@@ -2030,6 +2324,10 @@ static GOptionEntry * network_mysqld_proxy_plugin_get_options(chassis_plugin_con
 		{ "no-proxy",                 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, NULL, "don't start the proxy-module (default: enabled)", NULL },
 		
 		{ "proxy-pool-no-change-user", 0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, NULL, "don't use CHANGE_USER to reset the connection coming from the pool (default: enabled)", NULL },
+
+		{ "proxy-connect-timeout",    0, 0, G_OPTION_ARG_DOUBLE, NULL, "connect timeout in seconds (default: 2.0 seconds)", NULL },
+		{ "proxy-read-timeout",    0, 0, G_OPTION_ARG_DOUBLE, NULL, "read timeout in seconds (default: 8 hours)", NULL },
+		{ "proxy-write-timeout",    0, 0, G_OPTION_ARG_DOUBLE, NULL, "write timeout in seconds (default: 8 hours)", NULL },
 		
 		{ NULL,                       0, 0, G_OPTION_ARG_NONE,   NULL, NULL, NULL }
 	};
@@ -2045,6 +2343,9 @@ static GOptionEntry * network_mysqld_proxy_plugin_get_options(chassis_plugin_con
 	config_entries[i++].arg_data = &(config->lua_script);
 	config_entries[i++].arg_data = &(config->start_proxy);
 	config_entries[i++].arg_data = &(config->pool_change_user);
+	config_entries[i++].arg_data = &(config->connect_timeout_dbl);
+	config_entries[i++].arg_data = &(config->read_timeout_dbl);
+	config_entries[i++].arg_data = &(config->write_timeout_dbl);
 
 	return config_entries;
 }
@@ -2123,7 +2424,7 @@ int network_mysqld_proxy_plugin_apply_config(chassis *chas, chassis_plugin_confi
 G_MODULE_EXPORT int plugin_init(chassis_plugin *p) {
 	p->magic        = CHASSIS_PLUGIN_MAGIC;
 	p->name         = g_strdup("proxy");
-	p->version		= g_strdup(PACKAGE_VERSION);
+	p->version		= g_strdup(PLUGIN_VERSION);
 
 	p->init         = network_mysqld_proxy_plugin_new;
 	p->get_options  = network_mysqld_proxy_plugin_get_options;
