@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2009, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -50,6 +50,29 @@
 #ifndef WIN32
 #define closesocket(x) close(x)
 #endif
+
+#ifdef _WIN32
+#define E_NET_CONNRESET WSAECONNRESET
+#define E_NET_CONNABORTED WSAECONNABORTED
+#define E_NET_WOULDBLOCK WSAEWOULDBLOCK
+#define E_NET_INPROGRESS WSAEINPROGRESS
+#else
+#define E_NET_CONNRESET ECONNRESET
+#define E_NET_CONNABORTED ECONNABORTED
+#define E_NET_INPROGRESS EINPROGRESS
+#if EWOULDBLOCK == EAGAIN
+/**
+ * some system make EAGAIN == EWOULDBLOCK which would lead to a 
+ * error in the case handling
+ *
+ * set it to -1 as this error should never happen
+ */
+#define E_NET_WOULDBLOCK -1
+#else
+#define E_NET_WOULDBLOCK EWOULDBLOCK
+#endif
+#endif
+
 /**
  * create a new event-op
  *
@@ -81,7 +104,7 @@ void chassis_event_op_apply(chassis_event_op_t *op, struct event_base *event_bas
 	switch (op->type) {
 	case CHASSIS_EVENT_OP_ADD:
 		event_base_set(event_base, op->ev);
-		event_add(op->ev, NULL);
+		event_add(op->ev, op->tv);
 		break;
 	case CHASSIS_EVENT_OP_UNSET:
 		g_assert_not_reached();
@@ -89,6 +112,62 @@ void chassis_event_op_apply(chassis_event_op_t *op, struct event_base *event_bas
 	}
 }
 
+/**
+ * set the timeout 
+ *
+ * takes a deep-copy of the timeout as we have our own lifecycle independent of the caller 
+ */
+void
+chassis_event_op_set_timeout(chassis_event_op_t *op, struct timeval *tv) {
+	if (NULL != tv) {
+		op->_tv_storage = *tv;
+		op->tv = &(op->_tv_storage);
+	} else {
+		op->tv = NULL;
+	}
+}
+
+void chassis_event_add_with_timeout(chassis *chas, struct event *ev, struct timeval *tv) {
+	chassis_event_op_t *op = chassis_event_op_new();
+	gssize ret;
+
+	op->type = CHASSIS_EVENT_OP_ADD;
+	op->ev   = ev;
+	chassis_event_op_set_timeout(op, tv);
+
+	g_async_queue_lock(chas->threads->event_queue);
+	g_async_queue_push_unlocked(chas->threads->event_queue, op);
+
+	/* ping the event handler */
+	if (1 != (ret = send(chas->threads->event_notify_fds[1], C("."), 0))) {
+		int last_errno; 
+
+#ifdef _WIN32
+		last_errno = WSAGetLastError();
+#else
+		last_errno = errno;
+#endif
+
+		switch (last_errno) {
+		case EAGAIN:
+		case E_NET_WOULDBLOCK:
+			/* that's fine ... */
+			g_debug("%s: send() to event-notify-pipe failed: %s (len = %d)",
+					G_STRLOC,
+					g_strerror(errno),
+					g_async_queue_length_unlocked(chas->threads->event_queue));
+			break;
+		default:
+			g_critical("%s: send() to event-notify-pipe failed: %s (len = %d)",
+					G_STRLOC,
+					g_strerror(errno),
+					g_async_queue_length_unlocked(chas->threads->event_queue));
+			break;
+		}
+	}
+	g_async_queue_unlock(chas->threads->event_queue);
+}
+
 /**
  * add a event asynchronously
  *
@@ -98,13 +177,7 @@ void chassis_event_op_apply(chassis_event_op_t *op, struct event_base *event_bas
  * @see network_mysqld_con_handle()
  */
 void chassis_event_add(chassis *chas, struct event *ev) {
-	chassis_event_op_t *op = chassis_event_op_new();
-
-	op->type = CHASSIS_EVENT_OP_ADD;
-	op->ev   = ev;
-	g_async_queue_push(chas->threads->event_queue, op);
-
-	send(chas->threads->event_notify_fds[1], C("."), 0); /* ping the event handler */
+	chassis_event_add_with_timeout(chas, ev, NULL);
 }
 
 GPrivate *tls_event_base_key = NULL;
@@ -116,7 +189,7 @@ GPrivate *tls_event_base_key = NULL;
  *
  * @see network_connection_pool_lua_add_connection()
  */
-void chassis_event_add_local(chassis G_GNUC_UNUSED *chas, struct event *ev) {
+void chassis_event_add_local_with_timeout(chassis G_GNUC_UNUSED *chas, struct event *ev, struct timeval *tv) {
 	struct event_base *event_base = ev->ev_base;
 	chassis_event_op_t *op;
 
@@ -128,12 +201,16 @@ void chassis_event_add_local(chassis G_GNUC_UNUSED *chas, struct event *ev) {
 
 	op->type = CHASSIS_EVENT_OP_ADD;
 	op->ev   = ev;
+	chassis_event_op_set_timeout(op, tv);
 
 	chassis_event_op_apply(op, event_base);
 	
 	chassis_event_op_free(op);
 }
 
+void chassis_event_add_local(chassis *chas, struct event *ev) {
+	chassis_event_add_local_with_timeout(chas, ev, NULL);
+}
 /**
  * handled events sent through the global event-queue 
  *
@@ -147,23 +224,45 @@ void chassis_event_handle(int G_GNUC_UNUSED event_fd, short G_GNUC_UNUSED events
 	struct event_base *event_base = event_thread->event_base;
 	chassis *chas = event_thread->chas;
 	chassis_event_op_t *op;
-	char ping[1024];
-	guint received = 0;
-	gssize removed;
 
-	while ((op = g_async_queue_try_pop(chas->threads->event_queue))) {
-		chassis_event_op_apply(op, event_base);
+	do {
+		char ping[1];
 
-		chassis_event_op_free(op);
+		g_async_queue_lock(chas->threads->event_queue);
+		if ((op = g_async_queue_try_pop_unlocked(chas->threads->event_queue))) {
+			gsize ret;
+			chassis_event_op_apply(op, event_base);
 
-		received++;
-	}
+			chassis_event_op_free(op);
+	       
+			if (1 != (ret = recv(event_thread->notify_fd, ping, 1, 0))) {
+				/* we failed to pull .'s from the notify-queue */
+				int last_errno; 
 
-	/* the pipe has one . per event, remove as many as we received */
-	while (received > 0 && 
-	       (removed = recv(event_thread->notify_fd, ping, MIN(received, sizeof(ping)), 0)) > 0) {
-		received -= removed;
-	}
+#ifdef WIN32
+				last_errno = WSAGetLastError();
+#else
+				last_errno = errno;
+#endif
+
+				switch (last_errno) {
+				case EAGAIN:
+				case E_NET_WOULDBLOCK:
+					/* that's fine ... */
+					g_debug("%s: recv() from event-notify-fd failed: %s",
+							G_STRLOC,
+							g_strerror(last_errno));
+					break;
+				default:
+					g_critical("%s: recv() from event-notify-fd failed: %s",
+							G_STRLOC,
+							g_strerror(last_errno));
+					break;
+				}
+			}
+		}
+		g_async_queue_unlock(chas->threads->event_queue);
+	} while (op); /* even if op is 'free()d' it still is != NULL */
 }
 
 /**
@@ -229,7 +328,7 @@ chassis_event_threads_t *chassis_event_threads_new() {
 	 */
 	if (0 != evutil_socketpair(AF_UNIX, SOCK_STREAM, 0, threads->event_notify_fds)) {
 		int err;
-#ifdef WIN32
+#ifdef _WIN32
 		err = WSAGetLastError();
 #else
 		err = errno;
@@ -242,6 +341,10 @@ chassis_event_threads_t *chassis_event_threads_new() {
 	threads->event_threads = g_ptr_array_new();
 	threads->event_queue = g_async_queue_new();
 
+	/* make both ends non-blocking */
+	evutil_make_socket_nonblocking(threads->event_notify_fds[0]);
+	evutil_make_socket_nonblocking(threads->event_notify_fds[1]);
+
 	return threads;
 }
 
@@ -299,26 +402,35 @@ void chassis_event_threads_add(chassis_event_threads_t *threads, chassis_event_t
  * @see chassis_event_handle()
  */ 
 int chassis_event_threads_init_thread(chassis_event_threads_t *threads, chassis_event_thread_t *event_thread, chassis *chas) {
-#ifdef WIN32
+#ifdef _WIN32
 	LPWSAPROTOCOL_INFO lpProtocolInfo;
 #endif
 	event_thread->event_base = event_base_new();
 	event_thread->chas = chas;
-#ifdef WIN32
+#ifdef _WIN32
 	lpProtocolInfo = g_malloc(sizeof(WSAPROTOCOL_INFO));
 	if (SOCKET_ERROR == WSADuplicateSocket(threads->event_notify_fds[0], GetCurrentProcessId(), lpProtocolInfo)) {
-		g_error("%s: Could not duplicate socket: %s (%d)", G_STRLOC, g_strerror(WSAGetLastError()), WSAGetLastError());
+		int _errno = WSAGetLastError();
+
+		g_critical("%s: Could not duplicate socket: %s (%d)", G_STRLOC, g_strerror(_errno), _errno);
+		g_free(lpProtocolInfo);
+		return -1;
 	}
 	event_thread->notify_fd = WSASocket(FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, FROM_PROTOCOL_INFO, lpProtocolInfo, 0, 0);
 	if (INVALID_SOCKET == event_thread->notify_fd) {
-		g_error("%s: Could not create duplicated socket: %s (%d)", G_STRLOC, g_strerror(WSAGetLastError()), WSAGetLastError());
+		int _errno = WSAGetLastError();
+
+		g_critical("%s: Could not create duplicated socket: %s (%d)", G_STRLOC, g_strerror(_errno), _errno);
+		g_free(lpProtocolInfo);
+		return -1;
 	}
 	g_free(lpProtocolInfo);
 #else
 	event_thread->notify_fd = dup(threads->event_notify_fds[0]);
-#endif
-#if 0
-	evutil_make_socket_nonblocking(event_thread->notify_fd);
+	if (-1 == event_thread->notify_fd) {
+		g_critical("%s: Could not create duplicated socket: %s (%d)", G_STRLOC, g_strerror(errno), errno);
+		return -1;
+	}
 #endif
 
 	event_set(&(event_thread->notify_fd_event), event_thread->notify_fd, EV_READ | EV_PERSIST, chassis_event_handle, event_thread);
@@ -345,7 +457,11 @@ void *chassis_event_thread_loop(chassis_event_thread_t *event_thread) {
 		timeout.tv_sec = 1;
 		timeout.tv_usec = 0;
 
-		g_assert(event_base_loopexit(event_thread->event_base, &timeout) == 0);
+		r = event_base_loopexit(event_thread->event_base, &timeout);
+		if (r == -1) {
+			g_critical("%s: leaving chassis_event_thread_loop early. event_base_loopexit() failed", G_STRLOC);
+			break;
+		}
 
 		r = event_base_dispatch(event_thread->event_base);
 
