@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -51,6 +51,7 @@
 #include "chassis-keyfile.h"
 #include "chassis-filemode.h"
 #include "chassis-options.h"
+#include "chassis-win32-service.h"
 
 #include "string-len.h"
 
@@ -106,6 +107,7 @@ int chassis_frontend_init_win32() {
 
 		return -1;
 	}
+	chassis_win32_invalid_parameter_handler_set(chassis_win32_invalid_parameter_handler_log);
 
 	return 0;
 #else
@@ -311,6 +313,19 @@ int chassis_frontend_init_plugin_dir(char **_plugin_dir, const char *base_dir) {
 
 #ifdef WIN32
 	plugin_dir = g_build_filename(base_dir, "bin", NULL);
+
+#elif defined(PLUGINDIR) && defined(EXEC_PREFIX)
+
+	if (chassis_path_string_is_parent_of(C(EXEC_PREFIX), C(PLUGINDIR))) {
+		const char *rel_plugin_dir = PLUGINDIR + sizeof(EXEC_PREFIX) - 1;
+
+		if (rel_plugin_dir[0] == G_DIR_SEPARATOR) rel_plugin_dir++; /* if plugindir starts with a /, skip it */
+
+		plugin_dir = g_build_filename(base_dir, rel_plugin_dir, NULL);
+	} else {
+		plugin_dir = g_strdup(PLUGINDIR);
+	}    
+
 #else
 	plugin_dir = g_build_filename(base_dir, "lib", PACKAGE, "plugins", NULL);
 #endif
@@ -397,7 +412,7 @@ int chassis_frontend_init_plugins(GPtrArray *plugins,
 			}
 	
 			if (keyfile) {
-				if (chassis_keyfile_to_options(keyfile, keyfile_section_name, config_entries)) {
+				if (FALSE == chassis_keyfile_to_options_with_error(keyfile, keyfile_section_name, config_entries, gerr)) {
 					return -1;
 				}
 			}
@@ -477,9 +492,11 @@ int chassis_options_set_cmdline_only_options(chassis_options_t *opts,
 int chassis_frontend_print_version() {
 	/* allow to pass down a build-tag at build-time which gets hard-coded into the binary */
 #ifndef CHASSIS_BUILD_TAG
-#define CHASSIS_BUILD_TAG PACKAGE_STRING
+#define CHASSIS_BUILD_TAG PACKAGE_VERSION
+	g_print("  chassis: %s" CHASSIS_NEWLINE, CHASSIS_BUILD_TAG);
+#else
+	g_print("  chassis: %s.%s" CHASSIS_NEWLINE, PACKAGE_VERSION, CHASSIS_BUILD_TAG);
 #endif
-	g_print("  chassis: %s" CHASSIS_NEWLINE, CHASSIS_BUILD_TAG); 
 	g_print("  glib2: %d.%d.%d" CHASSIS_NEWLINE, GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
 	g_print("  libevent: %s" CHASSIS_NEWLINE, event_get_version());
 
@@ -522,6 +539,20 @@ void chassis_frontend_print_lua_version() {
 	lua_close(L);
 }
 
+/**
+ * log the versions of the initialized plugins
+*/
+int chassis_frontend_log_plugin_versions(GPtrArray *plugins) {
+	guint i;
+
+	for (i = 0; i < plugins->len; i++) {
+		chassis_plugin *p = plugins->pdata[i];
+
+		g_critical("plugin %s %s started", p->name, p->version);
+	}
+
+	return 0;
+}
 
 int chassis_frontend_write_pidfile(const char *pid_file, GError **gerr) {
 	int fd;
