@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -35,6 +35,7 @@
 #include <stdio.h> /* for _getmaxstdio() */
 #endif
 #include <errno.h>
+#include <stdlib.h>
 
 #include "chassis-limits.h"
 
@@ -72,9 +73,18 @@ int chassis_fdlimit_set(gint64 max_files_number) {
 	max_files_number_set = _setmaxstdio(max_files_number);
 
 	if (-1 == max_files_number_set) {
+		g_critical("%s: failed to set the maximum number of open files to %"G_GINT64_FORMAT" for stdio: %s (%d)",
+				G_STRLOC,
+				max_files_number,
+				g_strerror(errno),
+				errno);
 		return -1;
 	} else if (max_files_number_set != max_files_number) {
-		g_critical("%s: failed to increase the maximum number of open files for stdio: %s (%d)", G_STRLOC, g_strerror(errno), errno);
+		g_critical("%s: failed to increase the maximum number of open files to %"G_GINT64_FORMAT" for stdio: %s (%d)",
+				G_STRLOC,
+				max_files_number,
+				g_strerror(errno),
+				errno);
 		return -1;
 	}
 
