@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2009, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -28,6 +28,7 @@
 #else
 #include <stdlib.h> /* for realpath */
 #endif
+#include <string.h>
 #include "chassis-path.h"
 
 gchar *chassis_get_basedir(const gchar *prgname) {
@@ -120,9 +121,61 @@ gboolean chassis_resolve_path(const char *base_dir, gchar **filename) {
 	
 	g_debug("%s.%d: adjusting relative path (%s) to base_dir (%s). New path: %s", __FILE__, __LINE__, *filename, base_dir, new_path);
 
+	/**
+	 * We're going to rewrite the value of *filename, so we need to free the memory held by the pointer or it will lead to a memory leak.
+	 *
+	 * g_option_context_parse() is called after this call, so if it fails it will try to free the pointer. Since the pointer
+	 * was free'd and changed here, the storage(GOptionContext) which helds the pointer will not know about it and result in an access violation.
+	 * 
+	 * To fix this bug(#14665885) we need to change the chassis API. The temporary fix is to remove the free() and avoid the possible crash.
+	 * It's a non repeating leak so a minor drawback.
+	 */
+#if 0
 	g_free(*filename);
+#endif
+
 	*filename = new_path;
 	return TRUE;
 }
 
+/**
+ * Check if a path-string is parent of another
+ * @returns TRUE if the path-string is parent, FALSE if don't
+ */
+gboolean chassis_path_string_is_parent_of(const char *parent, gsize parent_len, const char *child, gsize child_len) {
+
+	if (child_len < parent_len)
+		return FALSE;
+
+	if(parent_len == 0)
+		return FALSE;
+
+	/* On Unix systems '/' is parent of all directories' */
+	if(0 == strcmp(parent, "/"))
+		return TRUE;
+
+	/* The path-string child can start with the same prefix string however the directory is not correct,
+	 * e.g. parent: /foo/bar , child: /foo/bar-foo.
+	 * To make the correct comparison we need to check if the parent path-string ends with the directory separator '/',
+	 * and if not, add it before doing the comparison
+	 */
+	if (parent[parent_len - 1] != G_DIR_SEPARATOR) {
+		/* g_strndup adds the trailing '\0' at the end of the new string */
+		char *temp_parent = g_strndup(parent, parent_len + 1);
+		temp_parent[parent_len] = G_DIR_SEPARATOR;
+
+		if (0 == strncmp(temp_parent, child, parent_len + 1)) {
+			g_free(temp_parent);
+			return TRUE;
+		}
+
+		g_free(temp_parent);
+		return FALSE;
+	}
+
+	if (0 == strncmp(parent, child, parent_len))
+		return TRUE;
+
+	return FALSE;
+}
 
