@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -336,11 +336,13 @@ int main_cmdline(int argc, char **argv) {
 	if (frontend->print_version) {
 #ifndef CHASSIS_BUILD_TAG
 #define CHASSIS_BUILD_TAG PACKAGE_STRING
+		g_print("%s" CHASSIS_NEWLINE, CHASSIS_BUILD_TAG);
+#else
+		g_print("%s.%s" CHASSIS_NEWLINE, PACKAGE_STRING, CHASSIS_BUILD_TAG);
 #endif
-		g_print("%s" CHASSIS_NEWLINE, CHASSIS_BUILD_TAG); 
 		chassis_frontend_print_version();
 	}
-	
+
 	/* add the other options which can also appear in the configfile */
 	opts = chassis_options_new();
 	chassis_frontend_set_chassis_options(frontend, opts);
@@ -359,7 +361,9 @@ int main_cmdline(int argc, char **argv) {
 	}
 
 	if (frontend->keyfile) {
-		if (chassis_keyfile_to_options(frontend->keyfile, "mysql-proxy", main_entries)) {
+		if (FALSE == chassis_keyfile_to_options_with_error(frontend->keyfile, "mysql-proxy", main_entries, &gerr)) {
+			g_critical("%s", gerr->message);
+
 			GOTO_EXIT(EXIT_FAILURE);
 		}
 	}
@@ -450,7 +454,7 @@ int main_cmdline(int argc, char **argv) {
 	}
 
 	/*
-	 * the MySQL Proxy should load 'admin' and 'proxy' plugins
+	 * the MySQL Proxy should load 'proxy' plugins
 	 */
 	if (!frontend->plugin_names) {
 		frontend->plugin_names = g_new(char *, 2);
@@ -562,10 +566,10 @@ int main_cmdline(int argc, char **argv) {
 		}
 	}
 
-	/* the message has to be _after_ the g_option_content_parse() to 
-	 * hide from the output if the --help is asked for
+	/* 
+	 * log the versions of all loaded plugins
 	 */
-	g_message("%s started", PACKAGE_STRING); /* add tag to the logfile (after we opened the logfile) */
+	chassis_frontend_log_plugin_versions(srv->modules);
 
 #ifdef _WIN32
 	if (chassis_win32_is_service()) chassis_win32_service_set_state(SERVICE_RUNNING, 0);
