@@ -1,5 +1,5 @@
 /* $%BEGINLICENSE%$
- Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
@@ -131,7 +131,9 @@ void network_socket_free(network_socket *s) {
 	network_address_free(s->dst);
 	network_address_free(s->src);
 
-	event_del(&(s->event));
+	if (s->event.ev_base) { /* if .ev_base isn't set, the event never got added */
+		event_del(&(s->event));
+	}
 
 	if (s->fd != -1) {
 		closesocket(s->fd);
@@ -355,11 +357,16 @@ network_socket_retval_t network_socket_connect(network_socket *sock) {
  * @see network_address_set_address()
  */
 network_socket_retval_t network_socket_bind(network_socket * con) {
+	/* WIN32:      int setsockopt(SOCKET s, int level, int optname, const char *optval, int optlen);
+	 * HPUX:       int setsockopt(int s,    int level, int optname, const void *optval, int optlen);
+	 * all others: int setsockopt(int s,    int level, int optname, const void *optval, socklen_t optlen);
+	 */
 #ifdef WIN32
-	char val = 1;	/* Win32 setsockopt wants a const char* instead of the UNIX void*...*/
+#define SETSOCKOPT_OPTVAL_CAST (const char *)
 #else
-	int val = 1;
+#define SETSOCKOPT_OPTVAL_CAST (void *)
 #endif
+
 	g_return_val_if_fail(con->fd < 0, NETWORK_SOCKET_ERROR); /* socket is already bound */
 	g_return_val_if_fail((con->socket_type == SOCK_DGRAM) || (con->socket_type == SOCK_STREAM), NETWORK_SOCKET_ERROR);
 
@@ -377,15 +384,24 @@ network_socket_retval_t network_socket_bind(network_socket * con) {
 
 		if (con->dst->addr.common.sa_family == AF_INET || 
 		    con->dst->addr.common.sa_family == AF_INET6) {
-			if (0 != setsockopt(con->fd, IPPROTO_TCP, TCP_NODELAY, &val, sizeof(val))) {
+			/* TCP_NODELAY  is int on unix, BOOL on win32 */
+			/* SO_REUSEADDR is int on unix, BOOL on win32 */
+#ifdef WIN32
+			BOOL val;
+#else
+			int val;
+#endif
+
+			val = 1;
+			if (0 != setsockopt(con->fd, IPPROTO_TCP, TCP_NODELAY, SETSOCKOPT_OPTVAL_CAST &val, sizeof(val))) {
 				g_critical("%s: setsockopt(%s, IPPROTO_TCP, TCP_NODELAY) failed: %s (%d)", 
 						G_STRLOC,
 						con->dst->name->str,
 						g_strerror(errno), errno);
 				return NETWORK_SOCKET_ERROR;
 			}
-			
-			if (0 != setsockopt(con->fd, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val))) {
+		
+			if (0 != setsockopt(con->fd, SOL_SOCKET, SO_REUSEADDR, SETSOCKOPT_OPTVAL_CAST &val, sizeof(val))) {
 				g_critical("%s: setsockopt(%s, SOL_SOCKET, SO_REUSEADDR) failed: %s (%d)", 
 						G_STRLOC,
 						con->dst->name->str,
@@ -394,12 +410,117 @@ network_socket_retval_t network_socket_bind(network_socket * con) {
 			}
 		}
 
+		if (con->dst->addr.common.sa_family == AF_INET6) {
+#ifdef IPV6_V6ONLY
+			/* disable dual-stack IPv4-over-IPv6 sockets
+			 *
+			 * ... if it is supported:
+			 * - Linux
+			 * - Windows
+			 * - Mac OS X
+			 * - FreeBSD
+			 * - Solaris 10 and later
+			 *
+			 * no supported on:
+			 * - Solaris 9 and earlier
+			 */
+
+			/* IPV6_V6ONLY is int on unix, DWORD on win32 */
+#ifdef WIN32
+			DWORD val;
+#else
+			int val;
+#endif
+
+			val = 0;
+			if (0 != setsockopt(con->fd, IPPROTO_IPV6, IPV6_V6ONLY, SETSOCKOPT_OPTVAL_CAST &val, sizeof(val))) {
+				g_critical("%s: setsockopt(%s, IPPROTO_IPV6, IPV6_V6ONLY) failed: %s (%d)", 
+						G_STRLOC,
+						con->dst->name->str,
+						g_strerror(errno), errno);
+				return NETWORK_SOCKET_ERROR;
+			}
+#endif
+		}
+
 		if (-1 == bind(con->fd, &con->dst->addr.common, con->dst->len)) {
-			g_critical("%s: bind(%s) failed: %s (%d)", 
+			gchar *address_copy;
+
+			/* binding failed so the address/socket is already being used
+			 * let's check if we can connect to it so we check if is being used 
+			 * by some app
+			 */
+			if (-1 == connect(con->fd, &con->dst->addr.common, con->dst->len)) {
+				g_debug("%s.%d: connect(%s) failed: %s (%d)", 
+					__FILE__, __LINE__,
+					con->dst->name->str,
+					g_strerror(errno), errno);
+				/* we can't connect to the socket so no one is listening on it. We need
+				 * to unlink it (delete the name from the file system) to be able to
+				 * re-use it.
+				 * network_address_free does the unlink, but to re-use it we need
+				 * to store the pathname associated with the socket before unlink it and
+				 * create a new socket with it.
+				 */
+				address_copy = g_strdup(con->dst->name->str);
+				con->dst->can_unlink_socket = TRUE;
+				network_address_free(con->dst);
+
+				con->dst = network_address_new();
+				network_address_set_address(con->dst, address_copy);
+
+				/* we can now free the address copy */
+				g_free(address_copy);
+
+				g_debug("%s: retrying to bind(%s)",
+						G_STRLOC,
+						con->dst->name->str);
+
+				/* let's bind again with the new socket */
+				if (-1 == bind(con->fd, &con->dst->addr.common, con->dst->len)) {
+					g_critical("%s: bind(%s) failed: %s (%d)", 
+						G_STRLOC,
+						con->dst->name->str,
+						g_strerror(errno), errno);
+
+					return NETWORK_SOCKET_ERROR;
+				}
+			} else {
+				g_critical("%s: bind(%s) failed: %s (%d)", 
 					G_STRLOC,
 					con->dst->name->str,
 					g_strerror(errno), errno);
-			return NETWORK_SOCKET_ERROR;
+
+				return NETWORK_SOCKET_ERROR;	
+			}
+		}
+
+		if (con->dst->addr.common.sa_family == AF_INET &&
+		    con->dst->addr.ipv4.sin_port == 0) {
+			struct sockaddr_in a;
+			socklen_t          a_len = sizeof(a);
+
+			if (0 != getsockname(con->fd, (struct sockaddr *)&a, &a_len)) {
+				g_critical("%s: getsockname(%s) failed: %s (%d)", 
+						G_STRLOC,
+						con->dst->name->str,
+						g_strerror(errno), errno);
+				return NETWORK_SOCKET_ERROR;
+			}
+			con->dst->addr.ipv4.sin_port  = a.sin_port;
+		} else if (con->dst->addr.common.sa_family == AF_INET6 &&
+		           con->dst->addr.ipv6.sin6_port == 0) {
+			struct sockaddr_in6 a;
+			socklen_t          a_len = sizeof(a);
+
+			if (0 != getsockname(con->fd, (struct sockaddr *)&a, &a_len)) {
+				g_critical("%s: getsockname(%s) failed: %s (%d)", 
+						G_STRLOC,
+						con->dst->name->str,
+						g_strerror(errno), errno);
+				return NETWORK_SOCKET_ERROR;
+			}
+			con->dst->addr.ipv6.sin6_port  = a.sin6_port;
 		}
 
 		if (-1 == listen(con->fd, 128)) {
